/*********************************************************************
*                SEGGER MICROCONTROLLER SYSTEME GmbH                 *
*        Solutions for real time microcontroller applications        *
**********************************************************************

File        : RTOSInit.c   (for Mitsubishi M16C)
Purpose     : Initializes and handles the hardware for embOS as far
              as required by embOS.
              Feel free to modify this file acc. to your
              target system.
---------------------------END-OF-HEADER------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*                    Configuration
*
**********************************************************************
*/

#define OS_CHIP 78403

#if OS_CHIP==78403
  #include "IO78403X.H"           /* adopt to your MCU */
#endif

#ifndef OS_CHIP
  #error Please select a chip (78404 or 78403).
  /* Supported chips are:  */
  /* 78404 for 78404x			 */
  /* 78403 for 78403x			 */
  /*
   *  If your chip is not yet supported, you need to modify this file
   * or get in touch with us (support@segger.com)
   *
	 */
#endif

/*  Frequency of target system CPU in Hz  */
#ifndef OS_FSYS
  #define OS_FSYS 32000000L
#endif

/* Select UART for embOSView.
   If you do not want (or can not due to hardware limitations) to dedicate
   a UART to embOSView, please define it to be -1
   Currently the standard code enables UART 2 as default
   and support UART2 only
*/
#ifndef OS_UART
  #define OS_UART 2
#endif    

#ifndef OS_BAUDRATE
  /*  It is nearly impossible to let the preprocessor calculate the */
  /*  settings of the dedicated baudrate generator, therfore this   */
  /*  define here is for reference only. Always check OS_COM_Init ! */
  /*  Predefined Baudrates 19200 or 38400 can be used here. All     */
  /*  Other baudrates require modification in OS_COM_Init()         */
  #define OS_BAUDRATE 38400L
#endif


/*  The following two defines are used for calculation of Timer     */
/*  reload value. They are not used to initialize the timer.        */
/*  If you use an other timer or change these values, please make   */
/*  sure, that the prescaler is set accordingly in OS_InitHW        */

#define OS_TIMER_PRESCALE 8       /* Prescaler used for embOS timer */
#define OS_SYS_CLK 1000           /* embOS tick frequency           */

/*  CAUTION: The timer prescaler has to be set higher than the CPU  */
/*           clock prescaler to enable propper reading of timer     */
/*           register value for profiling and embOS time info       */
/*           routines.                                              */
 
/*
**********************************************************************
*
*                    OS_InitHW
*
**********************************************************************

Initialize the hardware required for the OS to run. This will work
on any target hardware, but may have to be tailored a little
(regarding the clock frequency). Of course the same holds true
if for some reason you choose to use an other timer.
*/

void OS_InitHW(void) {
  #define TIMER_RELOAD OS_FSYS/OS_TIMER_PRESCALE/OS_SYS_CLK

  /* We initialize full speed (fxx/2). If this is not required      */
  /* done elsewhere, comment out the following line                 */
  STBC   =  0x00;      /* normal mode, fxx/2 (16 MHz)               */
  /*     Initialize timer for embOS                                 */
  /*     we use timer3 for this chip                                */
  PRM0  &= ~0xF0;      /* clear timer 3 prescaler bits              */
  PRM0  |=  0x10;      /* set timer 3 prescaler bits to fxx/8       */
  TMC0  |=  0x10;      /* timer 3 count in 16 bit mode              */
  CR30W  =  TIMER_RELOAD-1;
  CIC30  =  0x03;      /* Clear/enable interrupt priority 3         */
  MK0H  &= ~(1<<2);    /* Enable timer interupt                     */
  TMC0  |=  (1<<7);    /* timer 3 start                             */

#if (OS_UART >= 0)
  OS_COM_Init();       /* Initialize communication to embOS Viewer  */
#endif
}

/*
**********************************************************************
*
*                     Idle task  (OS_Idle)
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle task.
 This core loop can be changed, but:
 The idle task does not have a stack of its own, therefor no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {  /* Idle loop: No task is ready to exec  */
  for (;;);
}

/*
**************************************************************
*
*             Run-time error reaction  (OS_Error)
*
**************************************************************

 Run-time error reaction

 When this happens, a fatal error has occured and the kernel
 can not continue. In linux, the equivalent would be a
 "kernel panic"

 This routine can be modified to suit your needs ...
 E.g. a red LED could light up. When using an
 emulator, you may set a breakpoint here.
 In the release builds of the library (R), this routine is not required
 (as no checks are performed).
 In the stack check builds (S/SP), only error 120 may occur.
 In the debug builds(D/DP), all of the listed errors may occur.

  The following are the current errorcodes (V2.80) :
  (Please refer to the documentation for more info)
  
OS_ERR_STACK                          (120)
*** invalid or non-initialized data structures ***
OS_ERR_INV_TASK                       (128)
OS_ERR_INV_TIMER                      (129)
OS_ERR_INV_MAILBOX                    (130)
OS_ERR_INV_CSEMA                      (132)
OS_ERR_INV_RSEMA                      (133)

*** Using GetMail1 or PutMail1 or GetMailCond1 or PutMailCond1 on a non-1 byte mailbox  ***
OS_ERR_MAILBOX_NOT1                   (135)

***internal errors, please contact SEGGER Microcontrollersysteme ***
OS_ERR_MAILBOX_NOT_IN_LIST            (140)
OS_ERR_TASKLIST_CORRUPT               (142)


*** not matching routine calls or macro usage ***
OS_ERR_UNUSE_BEFORE_USE               (150)
OS_ERR_LEAVEREGION_BEFORE_ENTERREGION (151)
OS_ERR_LEAVEINT                       (152)
OS_ERR_DICNT                          (153)
OS_ERR_INTERRUPT_DISABLED             (154)

*** not a legal system call during interrupt ***
OS_ERR_ILLEGAL_IN_ISR                 (160)
*** not a legal system call during timer ***
OS_ERR_ILLEGAL_IN_TIMER               (161)

** double used data structures **
OS_ERR_2USE_TASK                      (170)
OS_ERR_2USE_TIMER                     (171)
OS_ERR_2USE_MAILBOX                   (172)
OS_ERR_2USE_BSEMA                     (173)
OS_ERR_2USE_CSEMA                     (174)
OS_ERR_2USE_RSEMA                     (175)

** Communication errors (OSCOM) **
OS_ERR_NESTED_RX_INT                  (180)
*/

void OS_Error(int ErrCode) {
  OS_EnterRegion();     /* Avoid further task switches */
  OS_DICnt =0;          /* Allow interrupts so we can communicate */
  OS_EI();
  OS_Status = ErrCode;
  while (OS_Status);
}

/*
**********************************************************************
*
*                    Get time [cycles]
*
**********************************************************************

    This routine is required for task-info via embOS Viewer.
    It returns the system time in clock cycles.
    When using an other timer, this routine has to be modified
*/

#define HW_TIMER TM3W
#define HW_TIMER_INTERRUPT_PENDING() (CIC30 & 0x80)

OS_U32 OS_GetTime_Cycles(void) {
  OS_U16 t_cnt   = TM3W;      
  OS_U32 time    = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = TM3W;
    if (t_cnt == TIMER_RELOAD)
      t_cnt =0;
    time++;
  }
  return (OS_FSYS/OS_TIMER_PRESCALE/OS_SYS_CLK)*time  + t_cnt;
}

/*
**********************************************************************
*
*                    OS_ConvertCycles2us
*
**********************************************************************

Purpose:
   Convert Cycles into micro seconds. (For profiling only)

   If you have a clock frequency which is not a multiple of 1 MHz,
   you may have to modify this routine in order to get proper
   diagonstics.
Please note:
   This routine is required for profiling only.
   It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= OS_TIMER_PRESCALE; /*  Eliminate effect of prescaler    */ 
  return Cycles/(OS_FSYS/1000000);
}

/*
**********************************************************************
*
*          Communication for embOSView   (UART 2)
*
**********************************************************************
*/
#if (OS_UART == 2)
/* #define BAUDDIVIDE (OS_FSYS/(OS_BAUDRATE*16L))-1  */

void OS_COM_Init(void) {
  MK1L  |= 0x1C;  /* Disable all UART related interrupts */
  CSIM2  = 0;     /* Clear CSI mode, just to be sure */
  ASIM2  = 0;     /* Stop any UART operation   */
#if OS_BAUDRATE == 19200
  BRGC2  = 0x3A;  /* set Baudrate to 19200 bps, when using 32,00 MHz */
#elif OS_BAUDRATE == 38400
  BRGC2  = 0x2a;  /* set Baudrate to 19200 bps, when using 32,00 MHz */
#else
  #error This OS_BAUDRATE setting not yet supported
#endif
  PM1    = 0xEF;  /* set P1.3 as input pin, P1.4 as output pin */
  PMC1   = 0x18;  /* set mode control register, enable P1.3 input and P1.4 output */
  ASIM2  = (1<<0) /* internal clock */
          +(0<<1) /* Rx interrupt on error */
          +(0<<2) /* 1 stop bit */
          +(1<<3) /* 8 bits data */
          +(0<<4) /* NO parity */
          +(3<<6) /* Tx / Rx enabled */
         ;
  SRIC2  = 0x03;  /* Enable/clear Rx interrupt, priority low */
  STIC2  = 0x03;  /* Enable/clear Tx interrupt, priority low */
  MK1L  &= ~(0x18);  /* Enable Rx Tx interrupt  */
}

void OS_COM_Send1(OS_U8 c) {
  TXS2 = c;
}

/* The interrupt routines */
#pragma function=near_func
#pragma codeseg(RCODE)

interrupt [0x2E] void OS_ISR_tx (void) { OS_OnTx(); }

/*  embOS Rx interrupt service routine uses IntStack switch    */
/*  Therefore a separate interupt handler routine is required  */

void OS_ISR_rxHandler(void) {
  OS_U8 RxError = ASIS2;
  OS_U8 Data    = RXB2;
  if (RxError == 0) {
    /* No errors occured, process data */
    OS_OnRx(Data);
  }
}

interrupt [0x2C] void OS_ISR_rx (void) {
  OS_EnterNestableInterrupt();  /*  We will enable interrupts  */
  OS_EnterIntStack();
	OS_ISR_rxHandler();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}


#elif (OS_UART ==-1)
  /* No communication routines */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_TxActive=0;  /* let OS know that transmitter is not busy  */
  }

#else
  #error Only UART2 (OS_UART=2) or None (OS_UART=-1) supported by now
#endif /* UART selection */

